﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using KSP;
using UnityEngine;

namespace IntuitiveManeuvers
{
    [KSPAddon(KSPAddon.Startup.Flight, false)]
    public class IntuitiveManeuvers : MonoBehaviour
    {
        // as far as I know, it's not possible for more than one maneuver gizmo to be visible at
        // any one time in the planetarium, but just in case, we maintain a list of gizmo handlers
        private List<ManeuverGizmoHandler> maneuverGizmoHandlers;

        public IntuitiveManeuvers()
        {
            this.maneuverGizmoHandlers = new List<ManeuverGizmoHandler>();
        }

        internal void Awake()
        {
        }

        internal void Start()
        {
        }

        internal void Update()
        {
            this.UpdateIntuitiveManeuverHandlersList();

            // iterate over the current handlers
            foreach (ManeuverGizmoHandler intuitiveManeuverHandler in this.maneuverGizmoHandlers)
                intuitiveManeuverHandler.OnUpdate();
        }

        internal void OnDestroy()
        {
            while (this.maneuverGizmoHandlers.Count > 0)
                this.maneuverGizmoHandlers[0].DeleteHandler();
        }

        internal void UpdateIntuitiveManeuverHandlersList()
        {
            // if any maneuver nodes exist for the current vessel, attach/detach from/to them
            if (FlightGlobals.ActiveVessel.patchedConicSolver.maneuverNodes != null)
            {
                foreach (ManeuverNode maneuverNode in FlightGlobals.ActiveVessel.patchedConicSolver.maneuverNodes)
                {
                    // if the maneuver node doesn't have a gizmo, don't attach
                    if (maneuverNode.attachedGizmo == null)
                        continue;

                    // only attach to new gizmos
                    bool alreadyHandled = false;
                    foreach (ManeuverGizmoHandler maneuverGizmoHandler in this.maneuverGizmoHandlers)
                    {
                        if (maneuverNode == maneuverGizmoHandler.ManeuverNode)
                        {
                            alreadyHandled = true;
                            break;
                        }
                    }

                    if (!alreadyHandled)
                    {
                        this.maneuverGizmoHandlers.Add(new ManeuverGizmoHandler(this, maneuverNode));
                    }
                }
            }
        }

        public void RemoveIntuitiveManeuverHandler(ManeuverGizmoHandler intuitiveManeuverHandler)
        {
            this.maneuverGizmoHandlers.Remove(intuitiveManeuverHandler);
        }
    }
}
